/*
troll.c - Slashdot trolls hack for RSStool

Copyright (c) 2006 NoisyB


This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#ifdef  HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <time.h>
#include "misc/net.h"
#include "misc/string.h"
#include "misc/rss.h"
#include "misc/misc.h"
#include "rsstool_defines.h"
#include "rsstool.h"
#include "rsstool_misc.h"
#include "troll.h"


#define TROLL_URL_S "http://slashdot.org"
#define TROLL_RSS_HOST_S "http://rss.slashdot.org"
#define TROLL_RSS_S "http://rss.slashdot.org/Slashdot/slashdot"
#define TROLL_REQ_OLDEST_FIRST_S "/comments.pl?threshold=-1&mode=flat&commentsort=0&startat=%d&sid=%s"
#define TROLL_REQ_NEWEST_FIRST_S "/comments.pl?threshold=-1&mode=flat&commentsort=1&startat=%d&sid=%s"
#define TROLL_ARTICLE_LINK_HOOK  "slashdot.org/article.pl?sid=%s"


//static st_net_t *n = NULL;
static int total_comment = 0;
static int funny_comment = 0;
static int troll_comment = 0;


int
troll_comment_headline (st_net_t *client, char *buf, char *comment_title, int *total_comment)
{
  *comment_title = 0;

  // START of comment title
  if (stristr (buf, "<div class=\"title\">"))
      {
        (*total_comment)++;

        strcpy (comment_title, buf);

        while (net_gets (client, buf, MAXBUFSIZE))
          {
            char *p = strchr (buf, '\n');

            if (p)
              *p = 0;

            strcat (comment_title, buf);

            // END of comment title
            if (stristr (buf, "</div>"))
              break;
          }
      }

  return 0;
}


int
troll_comment_details (st_net_t *client, char *buf, char *comment_details)
{
  // START of comment details
  if (stristr (buf, "<div class=\"details\">"))
    {
      strcpy (comment_details, buf);

      while (net_gets (client, buf, MAXBUFSIZE))
        {
          char *p = strchr (buf, '\n');

          if (p)
            *p = 0;

          strcat (comment_details, buf);

          // END of comment details
//          if (stristr (buf, "</div>"))
          if (stristr (buf, "</span>"))
            break;
        }
    }

  return 0;
}


int
troll_comment_desc (st_net_t *client, char *buf, char *comment_desc)
{
  if (stristr (buf, "<div id=\"comment_body_"))       // START of comment body
    {
      strncpy (comment_desc, buf, MAXBUFSIZE)[MAXBUFSIZE - 1] = 0;

      while (net_gets (client, buf, MAXBUFSIZE))
        {
          if (strlen (comment_desc) + strlen (buf) >= MAXBUFSIZE)
            return 0;

          strcat (comment_desc, buf);

          if (stristr (buf, "</div>"))    // END of comment body
            return 0;
         }
       return 0;
     }

  return -1;
}


#if 0
static const char *
get_sid (const char *str)
{
  char *p = "xx/xx/xx/xxxxxx";

  if (!(*str))
    return "";

  p = (char *) memmem2 (str, strlen (str), p, strlen (p), MEMCMP2_WCARD('x'));

  if (p)
    return p;

  return "";
}
#endif


int
troll_get_comment (const char *sid, int funny_flag)
{
  char buf[MAXBUFSIZE];
  char *p = NULL;
  int startat = 0;
  char comment_title[MAXBUFSIZE];
  char comment_details[MAXBUFSIZE];
  char comment_desc[MAXBUFSIZE];
  st_net_t *client = NULL;

#ifdef  DEBUG
  printf ("%s\n", sid);
#endif

  if (!(client = net_init (0)))
    {
      fprintf (stderr, "ERROR: client=net_init() failed\n");
      return -1;
    }

  total_comment = 0;
  funny_comment = 0;
  troll_comment = 0;
  *comment_title =
  *comment_details =
  *comment_desc = 0;

  while (total_comment > startat || !total_comment)
    {
      if (net_open (client, TROLL_URL_S, 80) != 0)
        {
          fprintf (stderr, "ERROR: could not log on to " TROLL_URL_S "\n");
          return -1;
        }

      startat = total_comment;        // startat 0 or last comment
      sprintf (buf, TROLL_URL_S TROLL_REQ_OLDEST_FIRST_S, startat, sid);
      p = net_build_http_request (buf, rsstool.user_agent, 0, NET_METHOD_GET, 0);
      net_write (client, (char *) p, strlen (p));

      // get link to article (only once per article)

      while (net_gets (client, buf, MAXBUFSIZE))
        {
          p = strchr (buf, '\n');
          if (p)
            *p = 0;

          troll_comment_headline (client, buf, comment_title, &total_comment);
          troll_comment_details (client, buf, comment_details);

          if (stristr (comment_title, "Flamebait)") ||
              stristr (comment_title, "Troll)") ||
              stristr (comment_title, "Offtopic)") ||
              stristr (comment_title, "Overrated)") ||
              (funny_flag && stristr (comment_title, "Funny)")) ||
              stristr (comment_title, "(Score:-1"))
            {
              if (funny_flag && stristr (comment_title, "Funny)"))
                funny_comment++;
              else
                troll_comment++;

              while (net_gets (client, buf, MAXBUFSIZE))
                if (!troll_comment_desc (client, buf, comment_desc))
                  break;

//              printf ("%s\n", comment_title);
//              printf ("%s\n", comment_details);
//              printf ("%s\n", comment_desc);

              // clean up
              strtriml (strtrimr (rsstool_strip_html (comment_details)));
              strtriml (strtrimr (rsstool_strip_html (comment_title)));
              strtriml (strtrimr (rsstool_strip_html (comment_desc)));

              rsstool_add_item_s (&rsstool,
                                  "Slashdot/Trolls",
                                  "http://rsstool.berlios.de",
                                  time (0),
                                  comment_details,
                                  comment_title,
                                  comment_desc);

              *comment_title =
              *comment_details =
              *comment_desc = 0;
            }
        }

      net_close (client);
    }

  net_quit (client);

  return troll_comment + funny_comment;
}


static const char *
get_link (const char *str)
{
  char buf[MAXBUFSIZE];
  const char *p = NULL;

  if (!(p = net_tag_get_value (str, "rdf:about")))
    return "";

#if 1
  strncpy (buf, p, MAXBUFSIZE)[MAXBUFSIZE - 1] = 0;
  strtrim_s (buf, "sid=", "&amp;");
#else
  p = get_sid (buf);
#endif

  troll_get_comment (buf, 0);

  return "";
}


st_tag_filter_t troll_item_filter[] = {
  {
    "item",
    get_link
  },
  {NULL, NULL}
};


int
troll_get_rss (st_rsstool_t *rt)
{
  (void) rt;
  st_rss_t *slashdot = NULL;
  int count = 0;
//  int i = 0;
  st_net_t *n = NULL;
  char buf[MAXBUFSIZE];
  const char *p = NULL;
  int cf = 0;

  p = net_http_get_to_temp (TROLL_RSS_S, rsstool.user_agent, 0);
  if (!p)
    {
      fprintf (stderr, "ERROR: failed to download " TROLL_RSS_S "\n");
      return -1;
    }

//  if (rss_demux (p) != 0)
//    return -1;

  slashdot = rss_open (p);
  if (!slashdot)
    return -1;

  count = rss_item_count (slashdot);
  if (!count)
    return -1;

  if (!(n = net_init (0)))
    {
      fprintf (stderr, "ERROR: n=net_init() failed\n");
      return -1;
    }

  if (net_open (n, TROLL_RSS_S, 80) != 0)
    {
      fprintf (stderr, "ERROR: could not log on to " TROLL_URL_S "\n");
      return -1;
    }

#ifdef  DEBUG
  for (i = 0; i < count; i++)
    {
      printf ("%s\n"
              "%s\n", 
        slashdot->item[i].title,
        slashdot->item[i].url);
    }
#endif

  p = net_build_http_request (TROLL_RSS_S, rsstool.user_agent, 0, NET_METHOD_GET, 0);
  net_write (n, (char *) p, strlen (p));

  while (net_gets (n, buf, MAXBUFSIZE))
    cf = net_tag_filter (buf, troll_item_filter, cf);

  net_quit (n);

  return 0;
}
